//
// Copyright 2008-2009 Johannes Hofmann <Johannes.Hofmann@gmx.de>
//
// This software may be used and distributed according to the terms
// of the GNU General Public License, incorporated herein by reference.

#include <sys/types.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>

#include "util.h"
#include "ExternalGui.H"

void
ExternalGui::button_cb(Fl_Widget* o, void* d) {
    ExternalGui *f = (ExternalGui*) d;
	const char *args[4];

	if (!f->gui_cmd || f->cmd_pid != -1)
		return;

	args[0] = "sh";
	args[1] = "-c";
	args[2] = f->gui_cmd;
	args[3] = 0;

	f->cmd_pid = pexecvp("sh", const_cast<char * const *>(args),
		&f->cmd_infp, &f->cmd_outfp);
	if (f->cmd_pid == -1) {
		fprintf(stderr, "Failed to start %s\n", f->gui_cmd);
		return;
	}

	fprintf(f->cmd_infp, "%s\n", f->iq->get_local_command());
	fflush(f->cmd_infp);
	Fl::add_fd(fileno(f->cmd_outfp), FL_READ, read_cb, d);
	f->gui_b->deactivate();
}

void
ExternalGui::read_cb(int fd, void* d) {
    ExternalGui *f = (ExternalGui*) d;
	char buf[2048];
	int status;

	if (fgets(buf, sizeof(buf), f->cmd_outfp) == NULL) {
		Fl::remove_fd(fileno(f->cmd_outfp));
		fclose(f->cmd_infp);
		fclose(f->cmd_outfp);
		f->cmd_infp = NULL;
		f->cmd_outfp = NULL;
		waitpid(f->cmd_pid, &status, 0);
		f->cmd_pid = -1;
		f->gui_b->activate();
	} else {
		char *newline = strchr(buf, '\n');
		if (newline)
			*newline = '\0';

		f->set_local_command(buf);
	}
}

ExternalGui::ExternalGui(int _x, int _y, int _w, int _h, const char *n) :
	FlimpText(_x, _y, _w, _h, n) {

	gui_cmd = NULL;
	cmd_infp = NULL;
	cmd_outfp = NULL;
	cmd_pid = -1;
	gui_b = new Fl_Button(0, 0, 40, 0, "&GUI");
	gui_b->callback((Fl_Callback*) button_cb, this);
	add_control(gui_b);
}

ExternalGui::~ExternalGui() {
	int status;

	if (cmd_pid != -1) {
		Fl::remove_fd(fileno(cmd_outfp));
		fclose(cmd_infp);
		fclose(cmd_outfp);
		waitpid(cmd_pid, &status, 0);
	}

	if (gui_cmd)
		free(gui_cmd);
}

void
ExternalGui::set_gui_data(const char *d) {
	if (gui_cmd)
		free(gui_cmd);
	gui_cmd = strdup(d);
}

void
ExternalGui::set_command(const char *c) {
	if (cmd_infp) {
		fprintf(cmd_infp, "%s\n", c);
		fflush(cmd_infp);
	}
	FlimpText::set_command(c);
}
