//
// Copyright 2007, 2008 Johannes Hofmann <Johannes.Hofmann@gmx.de>
//
// This software may be used and distributed according to the terms
// of the GNU General Public License, incorporated herein by reference.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <FL/Fl_Toggle_Button.H>

#include "FlimpSelect.H"

void
FlimpSelect::button_cb(Fl_Widget* o, void* d) {
    FlimpSelect *f = (FlimpSelect*) d;

	if (f->sel_b->value())
		f->select();
	else
		f->original();
}

void
FlimpSelect::ratio_cb(Fl_Widget* o, void* d) {
    FlimpSelect *f = (FlimpSelect*) d;
	Fl_Menu_* mw = (Fl_Menu_*)o;

	switch (mw->value()) {
		case 0:
			f->img->ratio = 0.0;
			break;
		case 1:
			f->img->ratio = (double) f->img->w() / (double) f->img->h();
			break;
		case 2:
			f->img->ratio = 4.0 / 3.0;
			break;
		case 3:
			f->img->ratio = 16.0 / 9.0;
			break;
	}
}

const char *
FlimpSelect::get_gui() {
	if (type == SelectImage::RECTANGLE)
		return "RectangleSelect";
	else if (type == SelectImage::POINT)
		return "PointSelect";
	else
		return "FlimpSelect";
}

void
FlimpSelect::cmd_cb(Fl_Widget* o, void* d) {
    FlimpSelect *f = (FlimpSelect*) d;

	f->set_gui_data(f->choice->value());
	f->choice->input()->mark(f->choice->input()->position()); // deselect text
	if (f->state == SELECTED) f->execute();
}

FlimpSelect::FlimpSelect(int _x, int _y, int _w, int _h,
	SelectImage::SelectType t, const char *n) :
	FlimpWidget(_x, _y, _w, _h, n) {

	type = t;
	command_format = NULL;

	choice = new Fl_Input_Choice(0, 0, 400, 0);
	choice->callback((Fl_Callback*) cmd_cb, this);
	Fl_Input *text = choice->input();
	text->color(FL_GRAY);
	text->when(FL_WHEN_ENTER_KEY_ALWAYS);
	add_control(choice);

	sel_b = new Fl_Toggle_Button(0, 0, 100, 0, "Select");
	sel_b->callback((Fl_Callback*) button_cb, this);
	add_control(sel_b);

	if (type == SelectImage::RECTANGLE) {
		ratio_c = new Fl_Choice(0, 0, 120, 0);
		ratio_c->add("free select", 0, (Fl_Callback*) ratio_cb, this);
		ratio_c->add("current ratio", 0, (Fl_Callback*) ratio_cb, this);
		ratio_c->add("4:3", 0, (Fl_Callback*) ratio_cb, this);
		ratio_c->add("16:9", 0, (Fl_Callback*) ratio_cb, this);
		ratio_c->value(0);
		add_control(ratio_c);
	}

	state = ORIGINAL;
	img->set_type(t);
}

void
FlimpSelect::set_gui_data(const char *d) {
	if (command_format)
		free(command_format);

	command_format = strdup(d);
	choice->value(command_format);
	choice->add(command_format);
}

void
FlimpSelect::to_local() {
	local_x = iq->global_trans->transform_x(orig_x, CoordinateTransform::TO_LOCAL);
	local_y = iq->global_trans->transform_y(orig_y, CoordinateTransform::TO_LOCAL);
	local_w = iq->global_trans->transform_scale_x(orig_w, CoordinateTransform::TO_LOCAL);
	local_h = iq->global_trans->transform_scale_y(orig_h, CoordinateTransform::TO_LOCAL);
}

void
FlimpSelect::to_orig() {
	orig_x = iq->global_trans->transform_x(local_x, CoordinateTransform::TO_ORIGINAL);
	orig_y = iq->global_trans->transform_y(local_y, CoordinateTransform::TO_ORIGINAL);
	orig_w = iq->global_trans->transform_scale_x(local_w, CoordinateTransform::TO_ORIGINAL);
	orig_h = iq->global_trans->transform_scale_y(local_h, CoordinateTransform::TO_ORIGINAL);
}

void
FlimpSelect::select() {
	img->get_rectangle(&local_x, &local_y, &local_w, &local_h);
	to_orig();
	img->set_type(SelectImage::NONE);
	state = SELECTED;
	execute();
}

void
FlimpSelect::original() {
	to_local();
	img->set_rectangle(local_x, local_y, local_w, local_h);
	img->set_type(SelectImage::RECTANGLE);
	state = ORIGINAL;
	execute();
}

void
FlimpSelect::execute() {
	if (state == SELECTED) {
		if (type == SelectImage::RECTANGLE) {
			snprintf(buf, sizeof(buf), command_format,
				(int) rint(local_w), (int) rint(local_h),
				(int) rint(local_x), (int) rint(local_y));
		} else {
			snprintf(buf, sizeof(buf), command_format,
				(int) rint(local_x), (int) rint(local_y));
		}
	} else {
		snprintf(buf, sizeof(buf), "cat");
	}

	set_local_command(buf);
}

void
FlimpSelect::set_command(const char *c) {
	int _x, _y, _w, _h;

	if (command_format &&
		((type == SelectImage::RECTANGLE &&
		  sscanf(c, command_format, &_w, &_h, &_x, &_y) == 4) ||
		 (type == SelectImage::POINT &&
		  sscanf(c, command_format, &_x, &_y) == 2))) {

		local_x = (double) _x;
		local_y = (double) _y;
		local_w = (double) _w;
		local_h = (double) _h;
		to_orig();
		img->set_type(SelectImage::NONE);
		sel_b->value(1);
		state = SELECTED;
	} else {
		img->set_type(type);
		sel_b->value(0);
		state = ORIGINAL;
	}
}
