//
// Copyright 2007 Johannes Hofmann <Johannes.Hofmann@gmx.de>
//
// This software may be used and distributed according to the terms
// of the GNU General Public License, incorporated herein by reference.

#ifndef FlimpWidget_H
#define FlimpWidget_H

#include <FL/Fl.H>
#include <FL/Fl_Pack.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Tabs.H>
#include <FL/Enumerations.H>

#include "Fl_Huge_Scroll.H"
#include "SelectImage.H"
#include "ImageQueue.H"

class FlimpWidget : public Fl_Group {
	private:
		Fl_Tabs *tabs;
		static double scroll_center_x;
		static double scroll_center_y;
		static double scroll_center_pos_x;
		static double scroll_center_pos_y;
		char * filename;

		void adjust_scroll();
		void store_visible_center();
		static void user_scroll_callback(void *d);
		Fl_Pack *ctrl;

	protected:
		SelectImage *img;
		void add_control(Fl_Widget *w);

	public:
		ImageQueue *iq;
		Fl_Huge_Scroll *scroll;

		FlimpWidget(int _x, int _y, int _w, int _h, const char *n = NULL);
		virtual ~FlimpWidget();
		void set_tabs(Fl_Tabs *t) {tabs = t; };
		int handle(int event);
		const char *get_name() {return label();};
		virtual const char *get_gui() {return "";};
		virtual const char *get_gui_data() {return "";};
		virtual void set_gui_data(const char *d) {};
		virtual void set_command(const char *c) {};
		void set_local_command(const char *c);
		int load(const char *filename);
		int reload();
		void up();
		void down();
		static void process_callback(void *d);
};

#endif
