//
// Copyright 2007 Johannes Hofmann <Johannes.Hofmann@gmx.de>
//
// This software may be used and distributed according to the terms
// of the GNU General Public License, incorporated herein by reference.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <png.h>

extern "C"
{
#include <jpeglib.h>
}

#include "ImageInfo.H"

ImageInfo::ImageInfo() {
	width = 0;
	height = 0;
	format = UNKNOWN;
}

int
ImageInfo::load(const char *file) {
	FILE *fp = fopen(file, "r");
	unsigned char header[16];
	char line[1024];

	format = UNKNOWN;
	width = 0;
	height = 0;

	if (fp == NULL) {
		perror("fopen");
		return 1;
	}
	
	if (fread(header, sizeof(header), 1, fp) != 1) {
		perror("fread1");
		fclose(fp);
		return 1;
	}

	rewind(fp);

	if (memcmp(header, "GIF87a", 6) == 0 ||
		memcmp(header, "GIF89a", 6) == 0) {
		format = GIF;
		width = header[6]; width += header[7] << 8;
		height = header[8]; height += header[9] << 8;

	} else if (header[0] == 'P' && header[1] >= '1' && header[1] <= '7') {
		char *blank;

		while (fgets(line, sizeof(line), fp) != NULL && !isdigit(line[0])){}
		width = (int) atof(line);
		blank = strchr(line, ' ');
		if (blank) {
			blank++;
			height = (int) atof(blank);
		} else {
			fprintf(stderr, "error parsing PNM header\n");
		}
		format = PNM;
		
	} else if (png_sig_cmp(header, 0, 8) == 0) {
		png_structp pp = png_create_read_struct(PNG_LIBPNG_VER_STRING,
			NULL, NULL, NULL);
		png_infop info = png_create_info_struct(pp);

		png_init_io(pp, fp);
		png_read_info(pp, info);
		width = (int) info->width;
		height = (int) info->height;
		format = PNG;
		png_destroy_read_struct(&pp, &info, NULL);
	} else if (memcmp(header, "\377\330\377", 3) == 0 &&
		header[3] >= 0xc0 && header[3] <= 0xef) {
		jpeg_decompress_struct dinfo;
		struct jpeg_error_mgr jerr;

		dinfo.err = jpeg_std_error(&jerr);
		jpeg_create_decompress(&dinfo);
		jpeg_stdio_src(&dinfo, fp);
		jpeg_read_header(&dinfo, 1);
		jpeg_calc_output_dimensions(&dinfo);
		width = dinfo.output_width; 
		height = dinfo.output_height;
		jpeg_destroy_decompress(&dinfo);
		format = JPEG;
	}

	fclose(fp);
	return 0;
}
