//
// Copyright 2007 Johannes Hofmann <Johannes.Hofmann@gmx.de>
//
// This software may be used and distributed according to the terms
// of the GNU General Public License, incorporated herein by reference.

#ifndef ImageQueue_H
#define ImageQueue_H

#include "CoordinateTransform.H"
#include "ImageInfo.H"

class ImageQueue {
		typedef void (Callback)(void *d);
	private:
		char *filename;
		ImageQueue *prev;
		ImageQueue *next;
		int id;
		CoordinateTransform *local_trans;
		ImageInfo *img_info;
		int processing_needed;
		Callback *proc_callback;
		void *proc_callback_data;
		char local_command[1024];
		char global_command[1024];
		int generation;
		int processed_generation;
		int processed_id;

		void update_global_trans();
		int convert(const char *str, char *buf, int buflen,
			CoordinateTransform::dir_t d, int strip);

	public:
		CoordinateTransform *global_trans;

		ImageQueue();
		~ImageQueue();
		void set_file(const char *p);
		const char *get_file() {return filename;};
		void set_local_command(const char *c);
		void set_global_command(const char *c);
		const char * get_local_command();
		const char * get_global_command() {return global_command;};
		int process();
		void append(ImageQueue* q);
		void insert(ImageQueue* q);
		void remove();
		void up();
		void down();
		void register_proc_callback(Callback *cb, void *d);
};

#endif
