//
// Copyright 2007 Johannes Hofmann <Johannes.Hofmann@gmx.de>
//
// This software may be used and distributed according to the terms
// of the GNU General Public License, incorporated herein by reference.

#include <stdio.h>
#include <math.h>

#include <FL/Fl.H>
#include <FL/fl_draw.H>

#include "SelectImage.H"

SelectImage::SelectImage(int _x, int _y, int _w, int _h) : Fl_Widget(_x, _y, _w, _h) {
	img = NULL;
	type = SelectImage::RECTANGLE;
	ratio = 0.0;
	sel_x = sel_y = 0;
	sel_w = sel_h = 10;
}

SelectImage::~SelectImage() {
	if (img)
		img->release();
}

int
SelectImage::load(const char *p) {
	if (img) img->release();

	img = Fl_Shared_Image::get(p);

	if (!img) {
		type = SelectImage::NONE;
		return 1;
	} else {
		size(img->w(), img->h());
		redraw();
		return 0;
	}
}

void
SelectImage::draw() {
	if (img == NULL) {
		fl_color(FL_WHITE);
		fl_rectf(x(), y(), w(), h());
		return;
	}

	fl_push_clip(x(), y(), w(), h());
	img->draw(x(), y(), w(), h(), 0, 0);
	fl_color(FL_RED);
	fl_line_style(FL_DASH, 1);

	if (type == SelectImage::RECTANGLE)
		fl_rect(x() + sel_x, y() + sel_y, sel_w, sel_h);
	else if (type == SelectImage::POINT)
		fl_circle(sel_x + x(), sel_y + y(), 2);

	fl_line_style(0);
	fl_pop_clip();
}

int
SelectImage::handle(int e) {
	switch (e) {
		case FL_PUSH:
			sel_x = Fl::event_x() - x();
			sel_y = Fl::event_y() - y();
			redraw();	
			return 1;
		case FL_DRAG:
			sel_w = Fl::event_x() - sel_x - x();
			sel_h = Fl::event_y() - sel_y - y();

			if (ratio > 0.0) {
				int tmp = (int) rint(sel_w / ratio);
				if (sel_h > tmp)
					sel_w = (int) rint(sel_h * ratio);
				else
					sel_h =tmp;
			}
			redraw();	
			return 1;
		case FL_KEYBOARD:
			{
				int key = Fl::event_key();
				if (key == FL_Left)
					sel_x--;	
				else if (key == FL_Right)
					sel_x++;
				else if (key == FL_Up)
					sel_y--;
				else if (key == FL_Down)
					sel_y++;
				else
					return 0;

				redraw();
				return 1;
			}
		case FL_FOCUS:
			return 1;
			break;
		case FL_UNFOCUS:
			return 0;
			break;
		default:
			break;
	}

	return 0;
}

void
SelectImage::get_point(double *_x, double *_y) {
	*_x = (double) sel_x;
	*_y = (double) sel_y;
}

void
SelectImage::get_rectangle(double *_x, double *_y, double *_w, double *_h) {
	*_x = (double) sel_x;
	*_y = (double) sel_y;
	*_w = (double) sel_w;
	*_h = (double) sel_h;
}

void
SelectImage::set_rectangle(double _x, double _y, double _w, double _h) {
	sel_x = (int) rint(_x);
	sel_y = (int) rint(_y);
	sel_w = (int) rint(_w);
	sel_h = (int) rint(_h);
}
