/* 
 * Copyright 2007-2009 Johannes Hofmann <Johannes.Hofmann@gmx.de>
 *
 * This software may be used and distributed according to the terms
 * of the GNU General Public License, incorporated herein by reference.
 */

#include <stdlib.h>
#include <unistd.h>

#include "util.h"

pid_t
pexecvp(const char *file, char *const argv[], FILE **infp, FILE **outfp) {
	int p_in[2], p_out[2];
	pid_t pid;

	if (pipe(p_in) < 0)
		return -1;

	if (pipe(p_out) < 0) {
		close(p_in[0]);
		close(p_in[1]);
		return -1;
	}

	pid = fork();

	if (pid == -1) {       /* error */
		close(p_in[0]);
		close(p_in[1]);
		close(p_out[0]);
		close(p_out[1]);
		return -1;
	} else if (pid == 0) { /* child */
		close(p_in[1]);
		close(p_out[0]);

		if (p_in[0] != STDIN_FILENO) {
			dup2(p_in[0], STDIN_FILENO);
			close(p_in[0]);
		}
		if (p_out[1] != STDOUT_FILENO) {
			dup2(p_out[1], STDOUT_FILENO);
			close(p_out[1]);
		}

		execvp(file, argv);
		_exit(127);
	} else {               /* parent */ 
		close(p_in[0]);
		close(p_out[1]);

		if (infp) {
			*infp = fdopen(p_in[1], "w");
		} else {
			close(p_in[1]);
		}

		if (outfp) {
			*outfp = fdopen(p_out[0], "r");
		} else {
			close(p_out[0]);
		}

		return pid;
	}
}
